#include <Adafruit_NeoPixel.h>

// Digital IO pin connected to the button. This will be driven with a
// pull-up resistor so the switch pulls the pin to ground momentarily.
// On a high -> low transition the button press logic will execute.
#define BUTTON_PIN 0

#define PIXEL_PIN 4  // Digital IO pin connected to the NeoPixels. ()

#define PIXEL_COUNT 11  // Number of NeoPixels

#define BRIGHT 10      // brightness
#define MAXBRIGHT 251  // luminosité max
#define BRIGHTSTEP 75  // pas d'augmentation de la lluminosité

#define NUM_PATTERNS 8  // Number of patterns

// Declare our NeoPixel strip object:
Adafruit_NeoPixel strip(PIXEL_COUNT, PIXEL_PIN, NEO_GRB + NEO_KHZ800);
// Argument 1 = Number of pixels in NeoPixel strip
// Argument 2 = Arduino pin number (most are valid)
// Argument 3 = Pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
//   NEO_RGBW    Pixels are wired for RGBW bitstream (NeoPixel RGBW products)

boolean oldState = HIGH;
int mode = 0;      // Currently-active animation mode, 0-7
int swBright = 1;  //gestion brightness
int brightness = BRIGHT;

uint16_t randomHue;

unsigned long ms = millis();

void setup() {
  pinMode(BUTTON_PIN, INPUT_PULLUP);
  randomSeed(analogRead(0));  // Seed the random number generator with an analog reading
  strip.begin();              // Initialize NeoPixel strip object (REQUIRED)
  strip.show();               // Initialize all pixels to 'off'
  strip.setBrightness(BRIGHT);

  randomHue = random(65536);
}

void loop() {

  switch (mode) {  // Start the new animation...
    case 0:
      strip.setBrightness(brightness);
      rainbowFull(150);
      break;
    case 1:
      strip.setBrightness(brightness);
      rainbow(10);  // rainbow
      break;
    case 2:
      strip.setBrightness(brightness);
      if (millis() - ms > 10 * 1000L) {  // toutes les XX secondes on change la couleur
        ms = millis();
        randomHue = random(65536);
      }
      colorFill(strip.gamma32(strip.ColorHSV(randomHue)));
      break;
    case 3:
      strip.setBrightness(brightness);
      colorFill(strip.Color(255, 0, 0));  // full Red
      break;
    case 4: // chenille rouge
      strip.setBrightness(brightness);
      colorWipe(strip.Color(255, 0, 0), 40);
      colorWipe(strip.Color(0, 0, 0), 40);
      break;
    case 5:  //full blanc
      strip.setBrightness(MAXBRIGHT);         // full brightness
      colorFill(strip.Color(127, 127, 127));  // full White
      break;
    case 6:  // chenille blanche
      strip.setBrightness(MAXBRIGHT);
      colorWipe(strip.Color(127, 127, 127), 40);
      colorWipe(strip.Color(0, 0, 0), 40);
      break;
    case 7:
      brightness += BRIGHTSTEP;
      if (brightness > MAXBRIGHT) brightness = BRIGHT;
      mode = 0;
      break;
  }
}

boolean checkButt() {
  // Get current button state.
  boolean newState = digitalRead(BUTTON_PIN);
  boolean ex = false;

  // Check if state changed from high to low (button press).
  if ((newState == LOW) && (oldState == HIGH)) {
    // Short delay to debounce button.
    delay(20);
    // Check if button is still low after debounce.
    newState = digitalRead(BUTTON_PIN);
    if (newState == LOW) {                        // Yes, still low
      if (++mode > (NUM_PATTERNS - 1)) mode = 0;  // Advance to next mode, wrap around after #8
      ex = true;
    }
  }
  // Set the last-read button state to the old state.
  oldState = newState;
  return ex;
}

// Fill strip pixels one after another with a color. Strip is NOT cleared
// first; anything there will be covered pixel by pixel. Pass in color
// (as a single 'packed' 32-bit value, which you can get by calling
// strip.Color(red, green, blue) as shown in the loop() function above),
// and a delay time (in milliseconds) between pixels.
void colorWipe(uint32_t color, int wait) {
  for (int i = 0; i < strip.numPixels(); i++) {  // For each pixel in strip...
    strip.setPixelColor(i, color);               //  Set pixel's color (in RAM)
    strip.show();                                //  Update strip to match
    delay(wait);                                 //  Pause for a moment
    if (checkButt()) break;
  }
}

void colorFill(uint32_t color) {                 //allume toutes les LED avec 1 couleur
  for (int i = 0; i < strip.numPixels(); i++) {  // For each pixel in strip...
    strip.setPixelColor(i, color);               //  Set pixel's color (in RAM)
  }
  strip.show();  //  Update strip to match
  checkButt();
}

// Theater-marquee-style chasing lights. Pass in a color (32-bit value,
// a la strip.Color(r,g,b) as mentioned above), and a delay time (in ms)
// between frames.
void theaterChase(uint32_t color, int wait) {
  for (int a = 0; a < 10; a++) {   // Repeat 10 times...
    for (int b = 0; b < 3; b++) {  //  'b' counts from 0 to 2...
      strip.clear();               //   Set all pixels in RAM to 0 (off)
                                   // 'c' counts up from 'b' to end of strip in steps of 3...
      for (int c = b; c < strip.numPixels(); c += 3) {
        strip.setPixelColor(c, color);  // Set pixel 'c' to value 'color'
      }
      strip.show();  // Update strip with new contents
      delay(wait);   // Pause for a moment
    }
    if (checkButt()) break;
  }
}

// Rainbow cycle along whole strip. Pass delay time (in ms) between frames.
void rainbow(int wait) {
  // Hue of first pixel runs 3 complete loops through the color wheel.
  // Color wheel has a range of 65536 but it's OK if we roll over, so
  // just count from 0 to 3*65536. Adding 256 to firstPixelHue each time
  // means we'll make 3*65536/256 = 768 passes through this outer loop:
  for (long firstPixelHue = 0; firstPixelHue < 3 * 65536; firstPixelHue += 256) {
    for (int i = 0; i < strip.numPixels(); i++) {  // For each pixel in strip...
                                                   // Offset pixel hue by an amount to make one full revolution of the
                                                   // color wheel (range of 65536) along the length of the strip
                                                   // (strip.numPixels() steps):
      int pixelHue = firstPixelHue + (i * 65536L / strip.numPixels());
      // strip.ColorHSV() can take 1 or 3 arguments: a hue (0 to 65535) or
      // optionally add saturation and value (brightness) (each 0 to 255).
      // Here we're using just the single-argument hue variant. The result
      // is passed through strip.gamma32() to provide 'truer' colors
      // before assigning to each pixel:
      strip.setPixelColor(i, strip.gamma32(strip.ColorHSV(pixelHue)));
    }
    strip.show();  // Update strip with new contents
    delay(wait);   // Pause for a moment
    if (checkButt()) break;
  }
}

// Rainbow cycle along whole strip. Pass delay time (in ms) between frames.
void rainbowFull(int wait) {
  // Hue of first pixel runs 3 complete loops through the color wheel.
  // Color wheel has a range of 65536 but it's OK if we roll over, so
  // just count from 0 to 3*65536. Adding 256 to firstPixelHue each time
  // means we'll make 3*65536/256 = 768 passes through this outer loop:
  for (long firstPixelHue = 0; firstPixelHue < 3 * 65536; firstPixelHue += 256) {
    for (int i = 0; i < strip.numPixels(); i++) {  // For each pixel in strip...
                                                   // Offset pixel hue by an amount to make one full revolution of the
                                                   // color wheel (range of 65536) along the length of the strip
                                                   // (strip.numPixels() steps):
      // strip.ColorHSV() can take 1 or 3 arguments: a hue (0 to 65535) or
      //int pixelHue = firstPixelHue + (i * 65536L / strip.numPixels());
      int pixelHue = firstPixelHue;  //+ (i * 65536L / strip.numPixels());
      // optionally add saturation and value (brightness) (each 0 to 255).
      // Here we're using just the single-argument hue variant. The result
      // is passed through strip.gamma32() to provide 'truer' colors
      // before assigning to each pixel:
      strip.setPixelColor(i, strip.gamma32(strip.ColorHSV(pixelHue)));
    }
    strip.show();  // Update strip with new contents
    delay(wait);   // Pause for a moment
    if (checkButt()) break;
  }
}


// Rainbow-enhanced theater marquee. Pass delay time (in ms) between frames.
void theaterChaseRainbow(int wait) {
  int firstPixelHue = 0;           // First pixel starts at red (hue 0)
  for (int a = 0; a < 30; a++) {   // Repeat 30 times...
    for (int b = 0; b < 3; b++) {  //  'b' counts from 0 to 2...
      strip.clear();               //   Set all pixels in RAM to 0 (off)
                                   // 'c' counts up from 'b' to end of strip in increments of 3...
      for (int c = b; c < strip.numPixels(); c += 3) {
        // hue of pixel 'c' is offset by an amount to make one full
        // revolution of the color wheel (range 65536) along the length
        // of the strip (strip.numPixels() steps):
        int hue = firstPixelHue + c * 65536L / strip.numPixels();
        uint32_t color = strip.gamma32(strip.ColorHSV(hue));  // hue -> RGB
        strip.setPixelColor(c, color);                        // Set pixel 'c' to value 'color'
      }
      strip.show();                 // Update strip with new contents
      delay(wait);                  // Pause for a moment
      firstPixelHue += 65536 / 90;  // One cycle of color wheel over 90 frames
    }
    if (checkButt()) break;
  }
}
